<?php
namespace wp_isms_contact\includes;
defined('ABSPATH') or die( 'Access Forbidden!' );

class iSMSContact {

    private $admin_contact_options;
    public $isms_contact_process;

    function __construct() {
        add_action('admin_menu', array($this,'isms_contact_hook_to_menu') );
        
        add_action("admin_enqueue_scripts", array($this,"isms_contact_scripts_and_style"));
        add_action('wp_enqueue_scripts', array($this,"isms_contact_public_scripts_and_style"));

        $this->admin_contact_options = get_option( 'isms_contact_account_settings' );

        $this->isms_contact_process = new \wp_isms_contact\includes\iSMSContactProcess();

        add_action( 'wp_ajax_get_contact_list', array($this, 'get_form_list') );
        add_action( 'wp_ajax_nopriv_get_contact_list', array($this, 'get_form_list') );

        add_action( 'wp_ajax_get_mail_sent_list', array($this, 'get_mail_sent_list') );
        add_action( 'wp_ajax_nopriv_get_mail_sent_list', array($this, 'get_mail_sent_list') );

        add_action( 'wp_ajax_add_form', array($this, 'add_form') );
        add_action( 'wp_ajax_nopriv_add_form', array($this, 'add_form') );

        add_action( 'wp_ajax_update_form', array($this, 'update_form') );
        add_action( 'wp_ajax_nopriv_update_form', array($this, 'update_form') );

        add_action( 'init', array($this,'register_shortcodes'));

        add_action( 'wp_ajax_send_email', array($this, 'send_email') );
        add_action( 'wp_ajax_nopriv_send_email', array($this, 'send_email') );

    }


    function send_email(){
        $error = false;
        $response = array();
        //Mail settings
        $form_id        = $_POST['form-id'];
        $addedfields    = $_POST['added_fields'];
        $mail_to        = $_POST['mail_to'];
        $mail_from      = $_POST['mail_from'];
        $mail_header    = $_POST['mail_header'];
        $mail_subject   = $_POST['mail_subject'];
        $mail_body      = $_POST['mail_body'];
        $html_format    = 'Content-Type: text/html; charset=UTF-8';
        if($_POST['html_format'] == 0) {
            $html_format = "Content-Type: text/plain; charset=UTF-8";
        }
		$attachment = "";

        //Defaults
        $name           = $_POST['fname']." ".$_POST['lname'];
        $email          = filter_var($_POST['email'], FILTER_SANITIZE_EMAIL);
        $subject        = $_POST['subject'];
        $message        = $_POST['message'];
        $country_code   = $_POST['isms_contact_country_code'];
        $mobilefield    = $_POST['mobilefield'];
        $mobile         = $_POST['isms_contact_mobilefield_hidden'];
        

        $fields         = explode(",", $addedfields);
        $fields_array   = array();
        $post_field     = array();

        $form_messages = $this->isms_contact_process->get_db_row(ISMS_CONTACT_FORM_MESSAGE,'form_id',$form_id);
        $form_fields = $this->isms_contact_process->get_db_data_id(ISMS_CONTACT_FORM_FIELDS,'form_id',$form_id);
          
        foreach($form_fields as $field){
          
            if($field->field_type == 'tel') {
				
                if($_POST[$field->field_name] != "") {
					if(!preg_match("/^[0-9]{3}-[0-9]{4}-[0-9]{4}$/", $_POST[$field->field_name])) {
						 $response['message'] =  'fsdfsd'.$_POST[$field->field_name];
                        $error = true;
					}
                }
            }else if($field->field_type == 'url') {
                if($_POST[$field->field_name] != "") {
                    if (!filter_var($_POST[$field->field_name], FILTER_VALIDATE_URL)) {
                        $response['message'] = $form_messages->url_invalid;
                        $error = true;
                    } 
                }
            }else if($field->field_type == 'file') {
                if (!empty($_FILES[$field->field_name]["name"])) {
                    $target_dir = wp_upload_dir();
                    if ( ! function_exists( 'wp_handle_upload' ) ) {
                        require_once( ABSPATH . 'wp-admin/includes/file.php' );
                    }
                    $upload_overrides = array(
                        'test_form' => false
                    );
                
                    $allowed = explode(",",$field->field_accept);
                    
                    $filename = $_FILES[$field->field_name]['name'];
                  
                    $ext = pathinfo($filename, PATHINFO_EXTENSION);
                    if (!in_array($ext, $allowed)) {
                        $response['message'] = $form_messages->upload_file_type_error;
                        $error = true;
                    }else {
                        if ($_FILES[$field->field_name]["size"] > $field->field_limit) {
                            $response['message'] = $form_messages->upload_file_too_large;
                            $error = true;
                        }else {

                            $movefile = wp_handle_upload($_FILES[$field->field_name], $upload_overrides );
         
                            if ( $movefile && ! isset( $movefile['error'] ) ) {
                              $attachment = $movefile;
                            } else {
                                $response['message'] = $form_messages->upload_php_error;
                                $error = true;
                            }  
                        }
                    }
                }
            }
            
        }       

        if(!is_numeric($mobilefield)) {
            $response['message'] = $form_messages->mobile_invalid;
            $error = true;
        }
        
        if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
            $response['message'] = $form_messages->email_invalid;
            $error = true;
        } 

        if(!$error){
            foreach ($fields as $key => $field) {
                array_push($fields_array,'[isms_'.$field.']');
                array_push($post_field, $_POST[$field]);
            }
          

            $sent = array(
                'form_id'   => $form_id,
                'mobile'    => $mobile,
                'name'      => $name,
                'email'     => $email,
                'subject'   => $subject,
                'message'   => $message,
                'date'      =>date("Y-m-d h:i:sa")
            );
           
            $headers=array(
                $html_format,
                'From: '.$mail_from,
                $mail_header
                
                
            );
            $msubject = $this->format_mail($fields_array,$post_field,$mail_subject);
            $mbody = $this->format_mail($fields_array,$post_field,$mail_body);
            $mheaders = $this->format_mail($fields_array,$post_field,$headers);
            
            if(wp_mail( $mail_to, $msubject, $mbody, $mheaders,$attachment )){
                $save_data = $this->isms_contact_process->save_data($sent,ISMS_CONTACT_SENT);
                
                if($save_data) {
                    $response['status'] = $save_data;
                    $response['message'] = $form_messages->sent_successfully;
                    wp_send_json($response);
                }  
            }else {
                $response['status'] ="Failed to send";
                $response['message'] = $form_messages->failed_to_send;
            }
            
        }else {
             wp_send_json($response);
        }

        
    }

    function get_messages($id,$field) {
        return $this->isms_contact_process->get_db_row(ISMS_CONTACT_FORM_MESSAGE,'form_id',$id);

    }
   
    function register_shortcodes(){
        add_shortcode('isms-contact-form', array($this,'isms_contact_form_function'));
        add_shortcode('isms-field', array($this,'isms_field_function'));
    }

    function isms_field_function($atts, $content = null){
        extract(shortcode_atts(array(
            'ty'    => null, //input type
            'fr'    => null, //required field
            'fn'    => null, //field name
            'dv'    => null, //default value
            'ph'    => null, //place holder
            'fid'   => null, //field ID
            'fc'    => null, //field Class
            'fl'    => null, //field label
            'ffa'   => null, //file accept
            'nmin'  => null, //number min
            'nmax'  => null //number max
        ), $atts));

        ob_start();
        
        $req = "";

        if($fr){
            $req = "required";
        }
        
        $createfield = '<p><input type="'.$ty.'" '.$req.' name="'.$fn.'" id="'.$fid.'" class="'.$fc.'" ';
     
        if($ty == "textarea"){

            if($ph != null && $ph == $dv) {
                $createfield ='<textarea name="'.$fn.'" '.$req.' placeholder="'.$ph.'" id="'.$fid.'" class="'.$fc.'"></textarea>'; 
            }else {
                $createfield ='<textarea name="'.$fn.'" '.$req.' id="'.$fid.'" class="'.$fc.'">'.$dv.'</textarea>'; 
            }
           
        }else{

            if($ty != 'file' && $ty != 'submit' && $ty != 'checkbox' && $ty != 'radio'){

                if($ph != null && $ph == $dv) {
                    $createfield .='placeholder="'.$ph.'" ';
                }else {
                    $createfield .='value="'.$dv.'" ';
                }
            }
            
            if($ty == 'file'){
                $createfield .='accept="'.$ffa.'" ';
            }

            if($ty == 'submit'){
                $createfield .='value="'.$fl.'" ';  
            }

            if($ty == 'number'){
                if($ph != null && $ph == $dv) {
                    $createfield .='min="'.$nmin.'" max="'.$nmax.'" ';
                }else {
                   $createfield .='min="'.$nmin.'" max="'.$nmax.'" ';
                }  
            }

            if($ty == 'checkbox' || $ty == "radio"){
                $createfield .= "><label for='".$fn."'class='checkandradiolabel'>".$fl."</label>";  

            }else {
                $createfield .= '>';
            }   
        }

        echo $createfield;
        $output_string = ob_get_contents();

        ob_end_clean();
        return $output_string;
    }

    function isms_contact_form_function($atts, $content = null){
        extract(shortcode_atts(array(
          'id'      => null,
          'title'   => null, 
        ), $atts));
        
        ob_start();

        $form_data = $this->isms_contact_process->get_db_row(ISMS_CONTACT_FORM,'id',$id);
        $form_meta_data = $this->isms_contact_process->get_db_row(ISMS_CONTACT_FORM_META,'form_id',$id);

        echo '<form id="isms-contact-form-'.$id.'" class="isms-contact-form" method="post" enctype="multipart/form-data"><input type="hidden" name="html_format" value="'.$form_meta_data->html_format.'"/><input type="hidden" name="form-id" value="'.$id.'"/><input type="hidden" name="action" value="send_email"><input type="hidden" name="added_fields" value="'.$form_meta_data->fields.'"><input type="hidden" name="mail_to" value="'.$form_meta_data->mail_to.'"><input type="hidden" name="mail_from" value="'.$form_meta_data->mail_from.'"><input type="hidden" name="mail_header" value="'.$form_meta_data->mail_additional_header.'"><input type="hidden" name="mail_subject" value="'.stripslashes(base64_decode($form_meta_data->mail_subject)).'"><input type="hidden" name="mail_body" value="'.stripslashes(base64_decode($form_meta_data->mail_body)).'">';

            echo do_shortcode(stripslashes(base64_decode($form_data->form_data)));
            
        echo '</form><div class="isms-response-holder isms-hidden"></div>';

        $output_string = ob_get_contents();
        ob_end_clean();

        return $output_string;
    }

    function add_form() {
        $title          = $_POST['title'];
        $form           = $_POST['form-data'];
        $addedfields    = $_POST['addedfields'];
        $user           = wp_get_current_user();

        $mailto         = $_POST['email-to'];
        $mailfrom       = $_POST['email-from'];
        $mailsubject    = $_POST['email-subject'];
        $mailheaders    = $_POST['email-headers'];
        $mailbody       = $_POST['email-body'];
        $html_format    = 0;
        if($_POST['html-format'] != NULL) {
            $html_format    = 1;
        }
        
        
        if($title == ""){
            $title = "Untitled";
        }

        $form = array(
            'title'     => $title,
            'form_data' => base64_encode($form),
            'author'    => $user->user_login,      
            'date'      => date("Y-m-d h:i:sa")
        );

        $meta = array(
            'mail_to'       => $mailto,
            'mail_from'     => $mailfrom,
            'mail_subject'  => base64_encode($mailsubject),
            'mail_additional_header' => $mailheaders,
            'mail_body'     => base64_encode($mailbody),
            'fields'        => $addedfields,
            'html_format'   => $html_format
        );

        $messages = array (
            'mobile_invalid' => $_POST['mobile_invalid'],
            'sent_successfully' => $_POST['sent_successfully'],
            'failed_to_send' => $_POST['failed_to_send'],
            'referred_to_as_spam' => $_POST['referred_to_as_spam'],
            'upload_error' => $_POST['upload_error'],
            'upload_file_type_error' => $_POST['upload_file_type_error'],
            'upload_file_too_large' => $_POST['upload_file_too_large'],
            'upload_php_error' => $_POST['upload_php_error'],
            'tel_invalid' => $_POST['tel_invalid'],
        );
        
       
        
        $save_data = $this->isms_contact_process->save_form($form,$meta,$messages);
        
        if($save_data) {
           foreach($_POST['fields'] as $value) {
                $field_data = array (
                    'form_id'       => $save_data,
                    'field_type'    => $value['field_type'],
                    'field_name'    => $value['field_name'],
                    'field_min'     => $value['field_min'],
                    'field_max'     => $value['field_max'],
                    'field_accept'  => $value['field_accept'],
                    'field_limit'   => $value['field_limit'],
                    'is_required'   => $value['is_required'],
                );
                
                $this->isms_contact_process->save_data($field_data,ISMS_CONTACT_FORM_FIELDS);
            }
            wp_send_json($save_data);
        
        }else{
            wp_send_json("Failed to create form");
        }
    }

    function update_form() {
        global $wpdb;
        $title          = $_POST['title'];
        $form           = $_POST['form-data'];
        $form_id        = $_POST['form-id'];
        $addedfields    = $_POST['addedfields'];

        $mailto         = $_POST['email-to'];
        $mailfrom       = $_POST['email-from'];
        $mailsubject    = $_POST['email-subject'];
        $mailheaders    = $_POST['email-headers'];
        $mailbody       = $_POST['email-body'];
        $html_format    = $_POST['html-format'];
        
        if($title == ""){
            $title = "Untitled";
        }

        $form_data = array(
           'title'      => $title,
           'form_data'  =>base64_encode($form),     
           'shortcode'  => '[isms-contact-form id="'.$form_id.'" title="'.$title.'"]'
        );

        $meta = array(
            'mail_to'       => $mailto,
            'mail_from'     => $mailfrom,
            'mail_subject'  => base64_encode($mailsubject),
            'mail_additional_header' => $mailheaders,
            'mail_body'     => base64_encode($mailbody),
            'fields'        => $addedfields,
            'html_format'   => $html_format
        );

         $messages = array (
             'mobile_invalid' => $_POST['mobile_invalid'],
            'sent_successfully' => $_POST['sent_successfully'],
            'failed_to_send' => $_POST['failed_to_send'],
            'referred_to_as_spam' => $_POST['referred_to_as_spam'],
            'upload_error' => $_POST['upload_error'],
            'upload_file_type_error' => $_POST['upload_file_type_error'],
            'upload_file_too_large' => $_POST['upload_file_too_large'],
            'upload_php_error' => $_POST['upload_php_error'],
            'tel_invalid' => $_POST['tel_invalid'],
        );


        $update_form = $this->isms_contact_process->update_form($form_id,$form_data,$meta,$messages);
 
        if($update_form){
            $wpdb->delete(
                ISMS_CONTACT_FORM_FIELDS,
                [ 'form_id' => $form_id ],
                [ '%d' ]
            );
            foreach($_POST['fields'] as $value) {
                $field_data = array (
                    'form_id'       => $form_id,
                    'field_type'    => $value['field_type'],
                    'field_name'    => $value['field_name'],
                    'field_min'     => $value['field_min'],
                    'field_max'     => $value['field_max'],
                    'field_accept'  => stripslashes($value['field_accept']),
                    'field_limit'   => $value['field_limit'],
                    'is_required'   => $value['is_required'],
                );
                
                $this->isms_contact_process->save_data($field_data,ISMS_CONTACT_FORM_FIELDS);
            }
            wp_send_json($update_form);
        }else{
            wp_send_json("Faild to update.");
        }
    }
    
    function get_form_list() {
        $forms = $this->isms_contact_process->get_db_data(ISMS_CONTACT_FORM);
        $lst = array();

        if($forms) {
            foreach ($forms as $form) {
                $lst['data'][]  = array(
                    'title' => '<a open-ul="#form-list-'.$form->id.'" href="'.get_site_url().'/wp-admin/admin.php?page=isms-contact-update&id='.$form->id.'">'.$form->title.'</a>
                                <div class="form-list-action-holder"><ul class="form-list-action isms-hidden" id="form-list-'.$form->id.'"><li><a href="'.get_site_url().'/wp-admin/admin.php?page=isms-contact-update&id='.$form->id.'">Edit</a></li></ul></div>',
                    'shortcode' => $form->shortcode,
                    'author' => $form->author,
                    'date' => $this->isms_contact_process->time_elapsed_string($form->timestamp)
                );
            }
            wp_send_json($lst);

        }else{
            $lst['data'][]  = array(
                'title'     => "",
                'shortcode' => "",
                'author'    => "No data available",
                'date'      =>"",
            );
            wp_send_json($lst);
        }
    }

    function get_mail_sent_list() {
        $sents = $this->isms_contact_process->get_db_data(ISMS_CONTACT_SENT);
        $lst = array();

        if($sents) {
            foreach ($sents as $sent) {
                $lst['data'][]  = array(
                    'mail_subject' => $sent->mail_subject,
                    'mail_from' => $sent->mail_from,
                    'mail_to' => $sent->mail_to,
                    'body' => $sent->mail_body,
                    'mobile' => $sent->mobile,
                    'date' => $this->isms_contact_process->time_elapsed_string($sent->timestamp)
                );
            }
            wp_send_json($lst);

        }else{
            $lst['data'][]  = array(
                'mail_subject' => "",
                'mail_from' => "",
                'mail_to' => "No data available",
                'body' => "",
                'mobile' => "",
                'date' => ""
            );
            wp_send_json($lst);
        }
    }

    function isms_contact_hook_to_menu() {
        add_menu_page(
            'iSMS Contact Form',
            'iSMS Contact',
            'manage_options',
            'isms-contact',
             array( $this, 'isms_contact_list' ),'',6
        );

        add_submenu_page(
            'isms-contact',
            'iSMS Add new form',
            'Add new',
            'manage_options',
            'isms-contact-new',
            array( $this, 'isms_contact_new' ),''
        );

        add_submenu_page(
            'isms-contact',
            'iSMS Sent Email',
            'Sent Mail',
            'manage_options',
            'isms-contact-sent',
            array( $this, 'isms_contact_sent' ),''
        );

        add_submenu_page(
            'isms-contact',
            'iSMS Update form',
            '',
            'manage_options',
            'isms-contact-update',
            array( $this, 'isms_contact_update' ),''
        );
    }

    function isms_contact_list () { ?>
        <div class="isms-contact">
            <div class="row isms-contact">
                <div class="col-md-12 mt-3">
                    <div class="col-md-6">
                        <h3>Contact Forms  <a href="<?php echo get_site_url();?>/wp-admin/admin.php?page=isms-contact-new" class="btn-primary" id="add-new-form">Add New</a></h3>
                    </div>
                    <div class="col-md-6 search-form-holder">
                        <form action="" method="post">                            
                            <input type="text" name="s" id="search" value="<?php the_search_query(); ?>" />
                            <button class="btn btn-primary">Search</button>
                        </form>  
                    </div>
                </div>
            </div> 
            <div class="isms-divider"></div>

            <div class="row">
                <div class="col-md-12">
                    <form method="post">
                        <?php 
                            $wp_list_table = new iSMSContactTableList(); 
                            $wp_list_table->prepare_items();
                            $wp_list_table->display();
                        ?>
                    </form>
                </div>
            </div>
        </div>                   
    <?php
    
    }

    function isms_contact_new () {
        $this->isms_contact_template('add-new');
    }

    function isms_contact_update () {
        $this->isms_contact_template('update-form');
    }

    function isms_contact_sent () {?>
        <div class="isms-contact">
            <div class="row isms-contact">
                <div class="col-md-12 mt-3">
                    <div class="col-md-6">
                        <h3>
                        <?php if(isset($_REQUEST['formID'])){
                            $form = $this->isms_contact_process->get_db_row(ISMS_CONTACT_FORM,'id',$_REQUEST['formID']);
                            echo $form->title;

                        } else {
                            echo 'Sent Mail';
                        } ?>       
                        </h3>
                    </div>
                    <div class="col-md-6 search-form-holder">
                        <form action="" method="post">                            
                            <input type="text" name="s" id="search" value="<?php the_search_query(); ?>" />
                            <button class="btn btn-primary">Search</button>
                        </form>  
                    </div>
                </div>
            </div> 
            <div class="isms-divider"></div>

            <div class="row">
                <div class="col-md-12">
                    <form method="post">
                        <?php 
                            $wp_list_table = new iSMSContactTableList(); 

                            $wp_list_table->prepare_items();
                            $wp_list_table->display();
                        ?>
                    </form>
                </div>
            </div>
        </div> 

    <?php }

    function isms_contact_scripts_and_style($hook){
       
        if($hook == 'toplevel_page_isms-contact' || $hook == 'isms-contact_page_isms-contact-new' || $hook == 'isms-contact_page_isms-contact-update' || $hook == 'isms-contact_page_isms-contact-sent'){
           wp_enqueue_style("isms-contact-bootstrap", 'https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/css/bootstrap.min.css');
            wp_enqueue_style("isms-contact-datatable", 'https://cdn.datatables.net/1.10.20/css/jquery.dataTables.min.css');
            wp_enqueue_style("isms-contact-prefix", plugins_url('../assets/prefix/css/intlTelInput.css', __FILE__));
            wp_enqueue_style("isms-contact-style", plugins_url('../assets/css/ismscontactstyle.css', __FILE__));

            wp_enqueue_script("isms-contact-bootstrap",'https://stackpath.bootstrapcdn.com/bootstrap/4.4.1/js/bootstrap.min.js');
            wp_enqueue_script("isms-contact-datatables",'https://cdn.datatables.net/1.10.20/js/jquery.dataTables.min.js');
            wp_enqueue_script("isms-contact-prefix-js", plugins_url('../assets/prefix/js/intlTelInput.js', __FILE__));
            wp_enqueue_script("isms-contact-js", plugins_url('../assets/js/ismscontact.js', __FILE__));
            
            wp_localize_script('isms-contact-js', 'ismscontactajaxurl', array("scriptismscontact" => admin_url('admin-ajax.php')));

            wp_localize_script('isms-contact-js', 'ismscontactScript', array(
                'pluginsUrl' => plugin_dir_url( __FILE__ ),
            ));
        }
    }

    function isms_contact_public_scripts_and_style($hook){
        wp_enqueue_style("isms-contact-prefix", plugins_url('../assets/prefix/css/intlTelInput.css', __FILE__));
        wp_enqueue_style("isms-contact-style", plugins_url('../assets/public/css/ismscontactstyle.css', __FILE__));
        wp_enqueue_script('jquery');
        wp_enqueue_script("isms-contact-prefix-js", plugins_url('../assets/prefix/js/intlTelInput.js', __FILE__));

        wp_enqueue_script("isms-contact-js", plugins_url('../assets/public/js/ismscontact.js', __FILE__));
        wp_localize_script( 'isms-contact-js', 'isms_contact_public_ajax', array( "ajaxurl" => admin_url('admin-ajax.php') ) );

        wp_localize_script('isms-contact-js', 'ismscontactScript', array(
            'pluginsUrl' => plugin_dir_url( __FILE__ ),
        ));
    }

    private function format_mail($fields,$replace_with,$str) {
        return str_replace($fields,$replace_with,$str);
    }
    
    private function isms_contact_template($file) {
        include(dirname(__FILE__) . '/'.$file.'.php');
    }
}

?>